%  k-means method for Nonstationary Time Series
%  based on common trend (sample program)
%
%  by Norio Watanabe, Chuo University, JAPAN
%                                        11/26/2022
%                                        10/04/2021
%
%  Note: common trend is calculated by movie average method 
%        with triangle weight
%
%   *******************************************************
%   ***  Original KMT argorithm (KMT1.m) is for trends  ***
%   ***                                see Reference    ***
%   *******************************************************
% 
%  Reference:
%     A k-means method for trends of time series: An application to 
%     time series of COVID-19 cases in Japan
%          Japanese Journal of Statistics and Data Science, 2022
%
%  contents:
%     KMNTS1.m : main M-file using following functions
%           kmeansTrendNTS.m
%           CommonTrendNTS.m
%           CommonTrendMA.m
%           fun_dsmCT.m
%           fun_dsmT.m
%     KMNTS2.m : auxiliary program (sort and summarize)
%
clear
close all
MAL=8;    % Length of Movig Aerage (2*MAL-1) for CommonTrendMA 
K=2;      % number of clusters ( several values of K should be tried )
Lmax=10;  % max lag (depends on time series. 0 is also accepted)
JCmax=5;  % iteration number
          % set large, if not well classified becuase of sensitivity on
          % initial values
rand('seed',97) % for setting initial values (old style)

%%%%%%%%%%%%%%%%%%
%
% X: N x P matrix
%    P-variate nonstationary time sieries
%    N:length of series
%
fname='data0';
load(strcat(fname,'.mat'));%%% loading data matrix X
% load('data0.mat'); 
%
% data0.mat : Covid-19 cases in Japan
%             provided by NHK (Japan Broadcasting Corporation)
%
% or, for example
% X=load(strcat(fname,'.txt')):
%
%%%%%%%%%%%%%%%%%%
[N,P]=size(X);
[CMT,kc,Lag,R,Jmin]=kmeansNTS(X,K,JCmax,Lmax,MAL);
%%%%%%%%%%%%%%%%%%

NT=N-2*Lmax;
for k=1:K
    subplot(K,1,k)
    hold on
    for p=1:P
        n1=Lmax+1+Lag(p);
        n2=N-Lmax+Lag(p);
        if kc(p)==k
            y=X(:,p);
            plot([N-n2+1:N-n1+1],R(p)*y(n1:n2),'b:')
        end
    end
    plot([Lmax+1:N-Lmax],CMT(:,k),'r-','LineWidth',3)
    hold off
    box on
end
for k=1:K
    sum(kc==k)
end

Jmin

% save results ( used by KMNTS2.m )
save(strcat(fname,'_res'),'K','Lmax','Jmin','kc','Lag','R','CMT') 


